from os import path
from rasa.cli.utils                import get_validated_path
from rasa.model                    import get_model, get_model_subdirectories
from rasa.nlu.model                import Interpreter
#from rasa.shared.nlu.training_data import TrainingData, Message
from rasa.shared.nlu.interpreter   import RegexInterpreter, INTENT_MESSAGE_PREFIX
from rasa.shared.utils.cli         import print_success

import rasa.nlu.run

from rasa.shared.constants import (
    DOCS_BASE_URL,
    DEFAULT_ENDPOINTS_PATH,
    DEFAULT_CREDENTIALS_PATH,
    DEFAULT_ACTIONS_PATH,
    DEFAULT_MODELS_PATH,
)

FILE_NAME = "!nlu_debug_file.txt"
if not path.exists(FILE_NAME):
   print("file !nlu_debug_file.txt not exists!")
   exit()

model = get_validated_path("", "model", DEFAULT_MODELS_PATH)
model_path = get_model(model)
_, nlu_model = get_model_subdirectories(model_path)

##############################################################################
print_success("DEFAULT_MODELS_PATH:"+DEFAULT_MODELS_PATH)
print_success("model:"+model)
print_success("model_path:"+model_path)
##############################################################################
    
interpreter       = Interpreter.load(nlu_model, None)
regex_interpreter = RegexInterpreter()
maxPrintedIntents = 1  # число выводимых интентов

print_success("NLU model loaded. Type a message and press enter to parse it.")

###############################################################
# файл с результатами:
fout = open("!nlu_debug_file.out", 'w', encoding='utf-8')  
# Вывод на экран и в файл
def out (st):
    print(st)
    fout.write(st+"\n")

ar = []
with  open(FILE_NAME, 'r', encoding='utf-8') as f:     # читаем yml-файл
   txt  = f.read()
   ar   = txt.split("\n")

for t in ar:
    message = t.strip()      # .lower()  
    if len(message) == 0:
        continue
    if message.find('-----') >= 0 or message.find('======') >= 0:
        out(message)
        continue
                
    if message.startswith(INTENT_MESSAGE_PREFIX):
        loop = asyncio.get_event_loop()
        result = loop.run_until_complete(regex_interpreter.parse(message))
    else:
        result = interpreter.parse(message)

    msg = "%-20s > " % (t);
    msg += "intents: %s (%.3f" % (result['intent']['name'],result['intent']['confidence'])
    for k in result['entities']:
        msg += ", %s=%s" % (k['entity'],k['value'])
        if 'confidence_entity' in k:
            msg += " : %.2f" % ( k['confidence_entity'] )
    msg += ")"
    
    if 'intent_ranking' in result and maxPrintedIntents > 1:
        printedIntents = 1
        for intent in result['intent_ranking']:        
            if intent['name'] == result['intent']['name']:
                continue
            msg += (", %s (%.3f)" % (intent['name'],intent['confidence']))
            printedIntents = printedIntents + 1
            if printedIntents == maxPrintedIntents:
                break
    out(msg)

fout.close()

"""
{'text': 'зовите меня маша', 
'intent': {'id': 5846425762300176186, 'name': 'my_name', 'confidence': 0.9994565844535828}, 
'entities': [{'entity': '"name"', 'start': 12, 'end': 16, 'confidence_entity': 0.9980849027633667, 
              'value': 'маша', 'extractor': 'DIETClassifier'}], 
'intent_ranking': [{'id': 5846425762300176186, 'name': 'my_name', 'confidence': 0.9994565844535828}, 
{'id': -3183177557687976971, 'name': 'deny', 'confidence': 0.00016833457630127668}, 
{'id': 1916181083327477722, 'name': 'affirm', 'confidence': 9.996127482736483e-05}, 
{'id': 8690575099665227747, 'name': 'mood_unhappy', 'confidence': 9.281196253141388e-05}, 
{'id': -5610299248971807086, 'name': 'greet', 'confidence': 8.362063817912713e-05}, 
{'id': -923989697737468401, 'name': 'mood_great', 'confidence': 4.0231429011328146e-05}, 
{'id': 3160065119479831266, 'name': 'bot_challenge', 'confidence': 2.964206214528531e-05}, 
{'id': -4155911471353619244, 'name': 'goodbye', 'confidence': 2.8788572308258153e-05}], 
'response_selector': {'all_retrieval_intents': [], 
'default': {'response': {'id': None, 'responses': None, 'response_templates': None, 
'confidence': 0.0, 'intent_response_key': None, 
'utter_action': 'utter_None', 'template_name': 'utter_None'}, 'ranking': []}}}
"""